@@ -402,7 +402,7 @@ public:
     vers_info->interval.start= start;
     if (item->fix_fields_if_needed_for_scalar(thd, &item))
       return true;
-    bool error= get_interval_value(item, int_type, &vers_info->interval.step) ||
+    bool error= get_interval_value(thd, item, int_type, &vers_info->interval.step) ||
            vers_info->interval.step.neg || vers_info->interval.step.second_part ||
           !(vers_info->interval.step.year || vers_info->interval.step.month ||
             vers_info->interval.step.day || vers_info->interval.step.hour ||
@@ -421,7 +421,13 @@ public:
     vers_info->limit= limit;
     return !limit;
   }
+  bool vers_require_hist_part(THD *thd) const
+  {
+    return part_type == VERSIONING_PARTITION &&
+      thd->lex->vers_history_generating();
+  }
   int vers_set_hist_part(THD *thd);
+  void vers_check_limit(THD *thd);
   bool vers_setup_expression(THD *thd, uint32 alter_add= 0); /* Stage 1. */
   partition_element *get_partition(uint part_id)
   {
