@@ -222,14 +222,6 @@ static THD* threadpool_add_connection(CONNECT *connect, void *scheduler_data)
 {
   THD *thd= NULL;
 
-  DBUG_EXECUTE_IF("CONNECT_wait",
-  {
-    extern MYSQL_SOCKET unix_sock;
-    DBUG_ASSERT(unix_sock.fd >= 0);
-    while (unix_sock.fd >= 0)
-      my_sleep(1000);
-  });
-
   /*
     Create a new connection context: mysys_thread_var and PSI thread
     Store them in THD.
